"""Python API Wrapper for Linkedin.

More information about PAWL can be found at https://github.com/kylejb/PAWL.
"""

from .constants import __version__  # NOQA
from .linkedin import Linkedin  # NOQA
